/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                       Copyright (C) A. P. H.                           **
**                           BIALYSTOK 2007                               **
**                                                                        **
**                         MODULE UART2 Service                           **
**                                                                        **
**            Mikrokontroler ARM: LPC2362..LPC2368, LPC2372..LPC2378      **
**                                                                        **
**                     Wersja z dnia : 2007.03.25                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include <LPC23xx.h>
#include "uart3.h"
#include "types.h"
#include "system.h"
#include "irqvectors.h"
#include "serialconst.h"

#define UART3InterruptMask ( 1 << UART3_Interrupt )

#define InpSerialBuffSize     64
#define OutSerialBuffSize     64

#define PCUART3               ( ULONG ) 1 << 25

typedef struct {
                   USHORT	    RdDataPointer ;
                   USHORT     WrDataPointer ;
                   UCHAR      Buffer [ InpSerialBuffSize ] ;
                 } InpSerialBufferRecT ;

typedef struct {
                   USHORT	    RdDataPointer ;
                   USHORT     WrDataPointer ;
                   USHORT     BusyStatus ;
                   UCHAR      Buffer [ OutSerialBuffSize ] ;
                 } OutSerialBufferRecT ;


static volatile InpSerialBufferRecT UART3InpBuffer ;
static volatile OutSerialBufferRecT UART3OutBuffer ;


__irq void UART3IrqHandler ( void )
{
  UCHAR Data ;
  UCHAR IrqStatus ;
  /*-------------------------------------------*/
  for ( ; ; )
  {
    IrqStatus = ( UCHAR ) U3IIR ;
    if ( IrqStatus & 0x01 )
    {
      VICVectAddr = 0 ;
      return ;
    } /* if */ ;
    switch ( ( IrqStatus >> 1 ) & 0x03 )
    {
      case 1             :   /* 0 1 - transmitter */
        if ( UART3OutBuffer . RdDataPointer == UART3OutBuffer . WrDataPointer )
        {
          UART3OutBuffer . BusyStatus = 0 ;
          break ;
        } /* if */ ;
        Data = UART3OutBuffer . Buffer [ UART3OutBuffer . RdDataPointer ] ;
        UART3OutBuffer . RdDataPointer ++ ;
        if ( UART3OutBuffer . RdDataPointer >= OutSerialBuffSize )
          UART3OutBuffer . RdDataPointer = 0 ;
        U3THR = Data ;
        break ;
      case 2             :   /* 1 0 - reciever data */
        Data = U3RBR ;
        UART3InpBuffer . Buffer [ UART3InpBuffer . WrDataPointer ] = Data ;
        UART3InpBuffer . WrDataPointer ++ ;
        if ( UART3InpBuffer . WrDataPointer >= InpSerialBuffSize )
          UART3InpBuffer . WrDataPointer = 0 ;
        break ;
      default            :
        VICVectAddr = 0 ;
        return ;
    } /* switch */ ;
  } /* for */ ;
} /* UART3IrqHandler */


void UART3InputReset ( void )
{
  UART3InpBuffer . RdDataPointer = 0 ;
  UART3InpBuffer . WrDataPointer = 0 ;
} /* UART3InputReset */


static void UART3OutputReset ( void )
{
  UART3OutBuffer . RdDataPointer = 0 ;
  UART3OutBuffer . WrDataPointer = 0 ;
  UART3OutBuffer . BusyStatus = 0 ;
} /* UART3OutputReset */


static void UART3DisableInterrupt ( void )
{
  VICIntEnClr = UART3InterruptMask ;
} /* UART3DisableInterrupt */


static void UART3EnableInterrupt ( void )
{
  VICIntEnable = UART3InterruptMask ;
} /* UART3EnableInterrupt */


void UART3Send ( UCHAR Data )
{
  USHORT WStop ;
  /*-------------------------------------------*/
  UART3DisableInterrupt ( ) ;
  if ( ! UART3OutBuffer . BusyStatus )
  {
    U3THR = Data ;
    UART3OutBuffer . BusyStatus = 1 ;
    UART3EnableInterrupt ( ) ;
  } /* if ... */
  else
  {
    for ( ; ; )
    {
      WStop = UART3OutBuffer . WrDataPointer + 1 ;
      if ( WStop >= OutSerialBuffSize )
        WStop = 0 ;
      if ( WStop != UART3OutBuffer . RdDataPointer )
      {
        UART3OutBuffer . Buffer [ UART3OutBuffer . WrDataPointer ] = Data ;
        UART3OutBuffer . WrDataPointer = WStop ;
        UART3EnableInterrupt ( ) ;
        break ;
      } /* if ... */
      UART3EnableInterrupt ( ) ;
      NOP ( ) ;
      NOP ( ) ;
      NOP ( ) ;
      NOP ( ) ;
      NOP ( ) ;
      UART3DisableInterrupt ( ) ;
    } /* for */ ;
  } /* if ... else */ ;
} /* UART3Send */


void UART3SendString ( UCHAR * String )
{
  while ( * String )
  {
    UART3Send ( * String ) ;
    String ++ ;
  } /* while */ ;
} /* UART3SendString */


ULONG UART3DataPresent ( void )
{
  ULONG Result ;
  /*-------------------------------------------*/
  Result = FALSE ;
  UART3DisableInterrupt ( ) ;
  if ( UART3InpBuffer . RdDataPointer != UART3InpBuffer . WrDataPointer )
  {
    Result = TRUE ;
  } /* if */ ;
  UART3EnableInterrupt ( ) ;
  return ( Result ) ;
} /* UART3DataPresent */


UCHAR UART3GetData ( void )
{
  UCHAR Data ;
  /*-------------------------------------------*/
  UART3DisableInterrupt ( ) ;
  if ( UART3InpBuffer . RdDataPointer == UART3InpBuffer . WrDataPointer )
  {
    UART3EnableInterrupt ( ) ;
    return ( 0 ) ;
  } /* if */ ;
  Data = UART3InpBuffer . Buffer [ UART3InpBuffer . RdDataPointer ] ;
  UART3InpBuffer . RdDataPointer ++ ;
  if ( UART3InpBuffer . RdDataPointer >= InpSerialBuffSize )
    UART3InpBuffer . RdDataPointer = 0 ;
  UART3EnableInterrupt ( ) ;
  return ( Data ) ;
} /* UART3GetData */


UCHAR UART3Working ( void )
{
  UCHAR Data ;
  /*------------------------*/
  if ( UART3OutBuffer . BusyStatus )
    return FALSE ;
  Data = U3LSR ;
  if ( Data  & 0x40 )
    return ( TRUE ) ;
  else
    return ( FALSE ) ;
} /* UART3Working */


void UART3HardwInit( ULONG PeriphClk ,
                     UCHAR SerialMode ,
                     ULONG SerialSpeed )
{
  ULONG Fdiv ;
  ULONG Fmod ;
  ULONG LDivider ;
  ULONG HDivider ;
  /*-------------------------------------------*/
  UART3DisableInterrupt ( ) ;
  UART3InputReset ( ) ;
  UART3OutputReset ( ) ;
  Fdiv = ( PeriphClk / 16 ) / SerialSpeed ;
  Fmod = ( PeriphClk / 16 ) % SerialSpeed ;
  if ( ( Fmod << 1 ) >= SerialSpeed )
    Fdiv ++ ;
  LDivider = Fdiv % 256 ;
  HDivider = Fdiv / 256 ;
  U3FDR = 0x10 ;
  U3LCR = 0x80 ;
  U3DLL = LDivider ;
  U3DLM = HDivider ;
  U3LCR = SerialMode ;
  U3FCR = 0x07 ;
  VICVectAddr29  = ( unsigned long ) UART3IrqHandler ;
  VICVectCntl29  = 15 ;
  UART3EnableInterrupt ( ) ;
  U3IER = RecvDataInterruptEnable | TransmDataInterruptEnable ;
} /* UART3HardwInit */


void UART3InitEnvir ( void )
{
  PCONP |= PCUART3 ;
  PINSEL1 |= 0x003C0000 ;  // P2.25 -> TXD3  P2.26 -> RXD3
  UART3InputReset ( ) ;
  UART3OutputReset ( ) ;
} /* UART3InitEnvir */

